import 'package:firebase_core/firebase_core.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:ken/smeup/services/smeupLocalizationDelegate.dart';
import '/screens/main_screen.dart';
import '/services/change_notifier_service.dart';
import '/services/localization_delegate.dart';
import '/services/routing_service.dart';

import 'package:provider/provider.dart';

final RouteObserver<PageRoute> routeObserver = RouteObserver<PageRoute>();

Future<void> main() async {
  WidgetsFlutterBinding.ensureInitialized();
  await Firebase.initializeApp();
  runApp(const MyApp(
    key: Key('myApp'),
  ));
}

class MyApp extends StatefulWidget {
  const MyApp({Key key}) : super(key: key);

  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: ChangeNotifierService.getProviders(),
      child: MaterialApp(
        debugShowCheckedModeBanner: false,
        localizationsDelegates: const [
          LocalizationDelegate(),
          SmeupLocalizationDelegate(),
          GlobalMaterialLocalizations.delegate,
          GlobalWidgetsLocalizations.delegate,
          GlobalCupertinoLocalizations.delegate,
          DefaultCupertinoLocalizations.delegate
        ],
        supportedLocales: const [
          Locale('en', 'US'),
          Locale('it', 'IT'),
          Locale('fr', 'FR'),
        ],
        home: const MainScreen(), // AuthenticationWrapper(),
        onGenerateRoute: (RouteSettings settings) {
          return RoutingService.getRoutes(settings);
        },
        navigatorObservers: [routeObserver],
      ),
    );
  }
}
