import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:ken/smeup/models/smeup_fun.dart';
import 'package:ken/smeup/models/widgets/smeup_buttons_model.dart';
import 'package:ken/smeup/services/smeup_configuration_service.dart';
import 'package:ken/smeup/services/smeup_data_service.dart';
import 'package:ken/smeup/services/smeup_log_service.dart';
import 'package:ken/smeup/services/smeup_utilities.dart';
import 'package:ken/smeup/widgets/smeup_button.dart';
import 'package:ken/smeup/widgets/smeup_label.dart';
import 'package:ken/smeup/widgets/smeup_text_field.dart';
import 'package:ken/smeup/widgets/smeup_text_password.dart';
import '/models/environment_model.dart';
import '/services/localization_service.dart';
import '/widgets/remember_wrapper.dart';

class LoginScreen extends StatefulWidget {
  static const routeName = '/login-screen';
  final Function loggedInFunction;
  final _formKey = GlobalKey<FormState>();
  final _scaffoldKey = GlobalKey<ScaffoldState>();

  LoginScreen({Key key, this.loggedInFunction}) : super(key: key);

  static Future<bool> signoutFromBackend() async {
    try {
      // login to backend
      Map headers = {
        'Authorization': SmeupConfigurationService.getLocalStorage()
            .getString('authorization')
      };

      var smeupFun = SmeupFun.fromServiceName('*HTTP');
      final smeupServiceResponse = await SmeupDataService.invoke(smeupFun,
          httpServiceMethod: 'post',
          httpServiceUrl:
              '${SmeupConfigurationService.getHttpServiceEndpoint()}/disconnection',
          httpServiceContentType: 'application/x-www-form-urlencoded',
          httpServiceBody: {
            "user":
                SmeupConfigurationService.getLocalStorage().getString('user'),
            "pwd": SmeupConfigurationService.getLocalStorage()
                .getString('password'),
            "env": ''
          },
          headers: headers);

      if (!smeupServiceResponse.succeded) {
        SmeupLogService.writeDebugMessage(
            'The backend logout completed with errors',
            logType: LogType.error);
        return false;
      }

      bool remember =
          SmeupConfigurationService.getLocalStorage().getBool('remember') ??
              false;

      if (!remember) {
        SmeupConfigurationService.getLocalStorage().setString('user', '');
        SmeupConfigurationService.getLocalStorage().setString('password', '');
        SmeupConfigurationService.getLocalStorage()
            .setString('environmentCode', '');
      }

      SmeupConfigurationService.getLocalStorage()
          .setString('authorization', '');
    } catch (e) {
      return false;
    }
    return true;
  }

  @override
  _LoginScreenState createState() => _LoginScreenState();
}

class _LoginScreenState extends State<LoginScreen> {
  String _user = '';
  String _password = '';
  bool _remember = false;
  String _environmentCode = '';
  List<EnvironmentModel> environmentList =
      List<EnvironmentModel>.empty(growable: true);

  @override
  void initState() {
    _remember =
        SmeupConfigurationService.getLocalStorage().getBool('remember') ??
            false;
    if (_remember) {
      _user =
          SmeupConfigurationService.getLocalStorage().getString('user') ?? '';
      _password =
          SmeupConfigurationService.getLocalStorage().getString('password') ??
              '';
      _environmentCode = SmeupConfigurationService.getLocalStorage()
              .getString('environmentCode') ??
          '';
    } else {
      _user = '';
      _password = '';
      _environmentCode = '';
    }
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<List<EnvironmentModel>>(
      future: _getEnvironmentList(),
      builder: (BuildContext context,
          AsyncSnapshot<List<EnvironmentModel>> snapshot) {
        if (snapshot.connectionState == ConnectionState.waiting) {
          return Stack(
            children: [
              Container(
                //color: const Color(0xff06899b), punto
                color: Colors.white,
              ),
              const Center(
                child: SizedBox(
                    height: 30,
                    width: 30,
                    child: CircularProgressIndicator(
                      //color: Color(0xff06899b), punto
                      color: Colors.white,
                      //backgroundColor: Color(0xff53b9cc), punto
                      backgroundColor: Color(0xff06899b),
                    )),
              )
            ],
          );
        } else {
          return _getChildren();
        }
      },
    );
  }

  Widget _getChildren() {
    var containerHeight = SmeupUtilities.getDeviceInfo().safeHeight -
        SmeupButtonsModel.defaultHeight;

    return Theme(
      data: SmeupConfigurationService.getTheme(),
      child: Builder(
        builder: (BuildContext context) => Scaffold(
            //backgroundColor: const Color.fromRGBO(6, 140, 154, 20),
            backgroundColor: Colors.white,
            key: widget._scaffoldKey,
            body: Container(
              height: containerHeight,
              padding: const EdgeInsets.all(14),
              child: SingleChildScrollView(
                child: Form(
                    key: widget._formKey,
                    child: Column(children: [
                      Padding(
                        padding: const EdgeInsets.only(top: 35.0),
                        child: Center(
                            child: Image.asset(
                          "assets/images/appstore_1024.png",
                          height: 150,
                          width: 150,
                        )),
                      ),
                      const SizedBox(height: 20),
                      SmeupTextField(
                        widget._scaffoldKey,
                        widget._formKey,
                        label: 'User',
                        data: _user,
                        fontSize: 20,
                        //backColor: const Color.fromRGBO(6, 140, 154, 1000), punto
                        backColor: Colors.white,
                        //fontColor: Colors.white,
                        fontColor: const Color(0xff06899b),
                        //borderColor: Colors.white,
                        borderColor: const Color(0xff06899b),
                        //captionBackColor:
                        //const Color.fromRGBO(6, 140, 154, 1000),
                        captionBackColor: Colors.white,
                        //captionFontColor: Colors.white,
                        captionFontColor: const Color(0xff06899b),
                        clientValidator: (value) {
                          if (value.isEmpty) {
                            return LocalizationService.of(context)
                                .getLocalString('fieldRequired');
                          }
                          return null;
                        },
                        clientOnSave: (value) {
                          _user = value;
                        },
                        clientOnChange: (value) {
                          _user = value;
                        },
                      ),
                      const SizedBox(height: 12),
                      SmeupTextPassword(
                        widget._scaffoldKey,
                        widget._formKey,
                        label: 'Password',
                        data: _password,
                        fontSize: 20,
                        //fontColor: Colors.white, punto
                        fontColor: const Color(0xff06899b),
                        //backColor: const Color.fromRGBO(6, 140, 154, 1000),
                        backColor: Colors.white,
                        //captionBackColor:
                        //const Color.fromRGBO(6, 140, 154, 1000),
                        captionBackColor: Colors.white,
                        //captionFontColor: Colors.white,
                        captionFontColor: const Color(0xff06899b),
                        //iconColor: Colors.white,
                        iconColor: const Color(0xff06899b),
                        //buttonBackColor:
                        //const Color.fromRGBO(6, 140, 154, 1000),
                        buttonBackColor: Colors.white,
                        //borderColor: Colors.white,
                        borderColor: const Color(0xff06899b),
                        clientValidator: (value) {
                          if (value.isEmpty) {
                            return LocalizationService.of(context)
                                .getLocalString('fieldRequired');
                          }
                          return null;
                        },
                        clientOnSave: (value) {
                          _password = value;
                        },
                        clientOnChange: (value) {
                          _password = value;
                        },
                        showRules: false,
                      ),
                      const SizedBox(height: 10),
                      Row(
                        children: [
                          SmeupLabel(
                            widget._scaffoldKey,
                            widget._formKey,
                            [
                              LocalizationService.of(context)
                                  .getLocalString('rememberCredentials')
                            ],
                            width: 200,
                            align: Alignment.centerLeft,
                            //backColor:
                            //const Color.fromRGBO(6, 140, 154, 1000),
                            backColor: Colors.white,
                            //iconColor: Colors.white,
                            iconColor: const Color(0xff06899b),
                            //fontColor: Colors.white),
                            fontColor: const Color(0xff06899b),
                          ),
                          RememberWrapper(
                              key: const Key('loginForm'),
                              formKey: widget._formKey,
                              scaffoldKey: widget._scaffoldKey,
                              remember: _remember,
                              onCheckChanged: (remember) {
                                _remember = remember;
                              }),
                        ],
                      ),
                      const SizedBox(height: 30),
                      if (_environmentCode.isNotEmpty)
                        SmeupLabel(
                          widget._scaffoldKey,
                          widget._formKey,
                          [
                            LocalizationService.of(context)
                                .getLocalString('environment')
                          ],
                          //padding: const EdgeInsets.all(4000),
                          align: Alignment.centerLeft,
                          //fontColor: Colors.white,
                          fontColor: const Color(0xff06899b),
                          //backColor: const Color.fromRGBO(6, 140, 154, 1000),
                          backColor: Colors.white,
                          fontSize: 15,
                          height: 15,
                        ),
                      GestureDetector(
                        onTap: () {
                          showDialog(
                              context: context,
                              builder: (context) => _getEnvironmentsDialog());
                        },
                        child: Container(
                          decoration: BoxDecoration(
                            border: Border.all(
                              color: const Color(0xff06899b),
                            ),
                            borderRadius: BorderRadius.circular(9.0),
                            shape: BoxShape.rectangle,
                            //color: event[index].backgroundColor,
                          ),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: [
                              Expanded(
                                child: SmeupLabel(
                                  widget._scaffoldKey,
                                  widget._formKey,
                                  () {
                                    var labelSelect = LocalizationService.of(
                                            context)
                                        .getLocalString('selectAnEnvironment');
                                    List<String> res = [];
                                    if (environmentList.isEmpty) {
                                      res.add(labelSelect);
                                    } else {
                                      var envs = environmentList
                                          .where((element) =>
                                              element.code == _environmentCode)
                                          .toList();
                                      if (envs == null || envs.isEmpty) {
                                        res.add(labelSelect);
                                      } else {
                                        res.add(envs.first.description);
                                      }
                                    }
                                    return res;
                                  }(),
                                  align: Alignment.center,
                                  //backColor:
                                  //const Color.fromRGBO(6, 140, 154, 1000),
                                  backColor: Colors.white,
                                  height: 10,
                                  fontSize: 15,
                                  //fontColor: Colors.white),
                                  fontColor: const Color(0xff06899b),
                                ),
                              ),
                              const Icon(
                                Icons.keyboard_arrow_down,
                                //color: Colors.white,
                                color: Color(0xff06899b),
                                size: 32,
                              )
                            ],
                          ),
                        ),
                      ),
                    ])),
              ),
            ),
            floatingActionButton: SmeupButton(
              padding: const EdgeInsets.only(left: 40, right: 40, bottom: 10),
              //backColor: Colors.white,
              backColor: const Color(0xff06899b),
              //fontColor: const Color.fromRGBO(6, 140, 154, 10000000),
              fontColor: Colors.white,
              width: double.infinity,
              fontSize: 20,
              data: LocalizationService.of(context).getLocalString('continue'),
              clientOnPressed: () async {
                EnvironmentModel enviromentModel = environmentList.firstWhere(
                    (env) => env.code == _environmentCode,
                    orElse: () => null);
                _loginPressed(_user, _password, context, widget._formKey,
                    widget.loggedInFunction, _remember, enviromentModel);
              },
            ),
            floatingActionButtonLocation:
                FloatingActionButtonLocation.centerDocked,
            resizeToAvoidBottomInset: false),
      ),
    );
  }

  Future<List<EnvironmentModel>> _getEnvironmentList() async {
    environmentList.clear();
    String jsonFilePath =
        '${SmeupConfigurationService.jsonsPath}/environments.json';

    try {
      String jsonString = await rootBundle.loadString(jsonFilePath);

      for (var element in (jsonDecode(jsonString)['environments'] as List)) {
        environmentList.add(EnvironmentModel(element['code'],
            element['description'], element['url'], element['initialFun']));
      }

      SmeupLogService.writeDebugMessage(
          'Loaded environments from $jsonFilePath file');
    } catch (e) {
      SmeupLogService.writeDebugMessage(
          '_getEnvironmentList error: $e (${e.message ?? ''})',
          logType: LogType.error);
      SmeupLogService.writeDebugMessage(
          'error loding environments from $jsonFilePath file',
          logType: LogType.error);
    }

    return environmentList;
  }

  Widget _getEnvironmentsDialog() {
    return AlertDialog(
      title: Column(
        children: [
          Text(LocalizationService.of(context)
              .getLocalString('selectAnEnvironment')),
          const Divider()
        ],
      ),
      content: SingleChildScrollView(
        child: ListBody(
            children: environmentList
                .map((e) => Row(
                      children: [
                        Radio(
                            value: e.code,
                            groupValue: _environmentCode,
                            onChanged: (value) {
                              setState(() {
                                _environmentCode = value;
                              });
                              Navigator.of(context).pop();
                              //widget.onValueChange(value);
                            }),
                        Text(e.description)
                      ],
                    ))
                .toList()),
      ),
      actions: [
        SmeupButton(
          data: 'Annulla',
          clientOnPressed: () {
            Navigator.of(context).pop();
          },
        ),
      ],
    );
  }

  Future<void> _loginPressed(String user, String password, context, formKey,
      refreshFunction, bool remember, EnvironmentModel environmentModel) async {
    var isValid = formKey.currentState.validate();

    if (isValid) {
      if (environmentModel == null || environmentModel.code.isEmpty) {
        isValid = false;
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(
            content: Text(LocalizationService.of(context)
                .getLocalString('selectAnEnvironment')),
            backgroundColor: SmeupConfigurationService.getTheme().errorColor,
          ),
        );
      }
    }

    if (isValid) {
      formKey.currentState.save();

      final loggedIn =
          await _signIntoBackend(user, password, remember, environmentModel);

      if (!loggedIn) {
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(
            content: Text(
                LocalizationService.of(context).getLocalString('loginFailed')),
            backgroundColor:
                SmeupConfigurationService.getTheme().errorColor, //PUNTO
          ),
        );
      } else {
        refreshFunction();
      }
    }
  }

  Future<bool> _signIntoBackend(String user, String password, bool remember,
      EnvironmentModel environmentModel) async {
    try {
      // login to backend
      SmeupConfigurationService.saveAltServiceEndpoint(
          ALT_SERVICE_ENDPOINTS.DEFAULT, environmentModel.url);
      SmeupConfigurationService.setDefaultServiceEndpoint();
      SmeupConfigurationService.saveAltServiceEndpoint(
          ALT_SERVICE_ENDPOINTS.HTTP, environmentModel.url);
      SmeupConfigurationService.setHttpServiceEndpoint();

      var smeupFun = SmeupFun.fromServiceName('*HTTP');
      final smeupServiceResponse = await SmeupDataService.invoke(smeupFun,
          httpServiceMethod: 'post',
          httpServiceUrl: '${environmentModel.url}/authentication',
          httpServiceContentType: 'application/x-www-form-urlencoded',
          httpServiceBody: {"user": user, "pwd": password, "env": ''});

      if (smeupServiceResponse.succeded) {
        SmeupConfigurationService.getLocalStorage().setString(
            'authorization',
            smeupServiceResponse.result.data.headers["authorization"][0]
                .toString());
        SmeupConfigurationService.getLocalStorage()
            .setBool('remember', remember);
        SmeupConfigurationService.getLocalStorage().setString('user', user);
        SmeupConfigurationService.getLocalStorage()
            .setString('password', password);
        SmeupConfigurationService.getLocalStorage()
            .setString('environmentCode', environmentModel.code);
        SmeupConfigurationService.getLocalStorage()
            .setString('initialFun', environmentModel.initialFun);
      } else {
        return false;
      }
    } catch (e) {
      e.toString();
      return false;
    }
    return true;
  }
}
