import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:ken/smeup/services/smeup_configuration_service.dart';
import 'package:ken/smeup/services/smeup_default_data_service.dart';
import 'package:ken/smeup/services/smeup_json_data_service.dart';

import 'package:ken/smeup/services/smeup_log_service.dart';
import 'package:flutter/material.dart';
import 'package:ken/smeup/services/firestore_data_service.dart';
import '../services/transformers/firestore_transformer.dart';
import '../services/transformers/kokos_transformer.dart';
import '/screens/land_page.dart';
import '/screens/login_screen.dart';
import '/services/localization_service.dart';

import 'package:ken/smeup/models/authentication_model.dart';

import 'package:package_info/package_info.dart';

class MainScreen extends StatefulWidget {
  static const routeName = '/MainScreen';

  const MainScreen({Key key}) : super(key: key);

  @override
  _MainScreenState createState() => _MainScreenState();
}

class _MainScreenState extends State<MainScreen> {
  bool _isInitialized = false;

  @override
  void initState() {
    _initPackageInfo();
    super.initState();
  }

  Future<void> _initPackageInfo() async {
    final PackageInfo info = await PackageInfo.fromPlatform();
    setState(() {
      SmeupConfigurationService.packageInfoModel = info;
    });
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async => false,
      child: FutureBuilder<void>(
          future: _configureServices(context),
          builder: (BuildContext context, AsyncSnapshot<void> snapshot) {
            if (snapshot.connectionState == ConnectionState.waiting) {
              return _getWait();
            } else if (SmeupConfigurationService.getLocalStorage()
                        .getString('authorization') ==
                    null ||
                SmeupConfigurationService.getLocalStorage()
                    .getString('authorization')
                    .isEmpty) {
              return LoginScreen(
                loggedInFunction: () {
                  setState(() {});
                },
              );
            } else {
              return const LandPage();
            }
          }),
    );
  }

  Future<void> _configureServices(BuildContext context) async {
    if (!_isInitialized) {
      await SmeupConfigurationService.init(
        context,
        logLevel: LogType.debug,
        localizationService: LocalizationService.of(context),
        appBarImage: 'logo_100x100.png',
        authenticationModel: AuthenticationModel(
            managed: true, logoutFunction: LoginScreen.signoutFromBackend),
        customDataServices: {
          '*DEFAULT': SmeupDefaultDataService(transformer: KokosTransformer()),
          '*JSN': SmeupJsonDataService(
              transformer: FirestoreTransformer(),
              firestoreInstance: FirebaseFirestore.instance),
          'FS_00_01': SmeupFirestoreDataService(FirebaseFirestore.instance,
              transformer: FirestoreTransformer()),
        },
      );
      _isInitialized = true;
    }
  }

  Widget _getWait() {
    return Stack(
      children: [
        Container(
          color: const Color(0xff06899b),
        ),
        const Center(
          child: SizedBox(
              height: 30,
              width: 30,
              child: CircularProgressIndicator(
                color: Color(0xff06899b),
                backgroundColor: Color(0xff53b9cc),
              )),
        )
      ],
    );
  }
}
