import 'dart:convert';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:ken/smeup/models/smeup_fun.dart';
import 'package:ken/smeup/services/smeup_data_service.dart';
import 'package:ken/smeup/services/smeup_utilities.dart';
import 'package:ken/smeup/services/transformers/smeup_data_transformer_interface.dart';

enum JsonType { nul, exd, exb, tre, fbk }

class FirestoreTransformer implements SmeupDataTransformerInterface {
  @override
  transform(SmeupFun smeupFun, data) {
    var firestoreJsonTransformer = FirestoreJsonTransformer();
    JsonType type =
        firestoreJsonTransformer.getJsonType(smeupFun.fun['fun']['component']);

    return FirestoreJsonTransformer().transform(smeupFun, type, data);
  }
}

class FirestoreJsonTransformer {
  JsonType getJsonType(String typeAsString) {
    JsonType type;
    switch (typeAsString.toUpperCase()) {
      case 'EXD':
        type = JsonType.exd;
        break;
      case 'EXB':
        type = JsonType.exb;
        break;
      case 'TRE':
        type = JsonType.tre;
        break;
      case 'FBK':
        type = JsonType.fbk;
        break;
      default:
        type = JsonType.nul;
    }
    return type;
  }

  transform(SmeupFun smeupFun, JsonType type, dynamic data) {
    List<Map<String, dynamic>> docs = convertResponse(data);

    switch (type) {
      case JsonType.exd:
        Map<String, dynamic> jsonData;

        if (docs.isNotEmpty) {
          var firstDocData = docs[0];
          String jsonString = '';
          jsonString = firstDocData['json'];
          jsonString = SmeupUtilities.replaceDictionaryPlaceHolders(jsonString);
          jsonData = jsonDecode(jsonString);
        }

        return jsonData;

      case JsonType.exb:
      case JsonType.tre:
        dynamic res = SmeupDataService.getEmptyDataStructure();

        // columns
        var columns = getColumns(docs);

        // rows
        var rows = getRows(docs);

        res['columns'] = columns;
        res['rows'] = rows;
        res['type'] = 'SmeupTreeNode';

        return res;

      case JsonType.fbk:
        return data;
      default:
        return data;
    }
  }

  convertResponse(data) {
    if (data is QuerySnapshot<Map<String, dynamic>>) {
      return getDataFromQuerySnapshot(data);
    } else {
      return getDataFromDocumentSnapshot(data);
    }
  }

  List<Map<String, dynamic>> getDataFromQuerySnapshot(
      QuerySnapshot<Map<String, dynamic>> data) {
    var docs = List<Map<String, dynamic>>.empty(growable: true);

    for (var i = 0; i < data.docs.length; i++) {
      var docData = data.docs.toList()[i];

      Map doc = docData.data();
      doc['id'] = docData.id;
      docs.add(doc);
    }

    return docs;
  }

  List<Map<String, dynamic>> getDataFromDocumentSnapshot(
      DocumentSnapshot<Map<String, dynamic>> data) {
    return [data.data()];
  }

  getColumns(List<Map<String, dynamic>> docs) {
    var columns = List<Map>.empty(growable: true);
    if (docs.isNotEmpty) {
      var firstDocData = docs[0];
      Map doc = firstDocData;
      columns.addAll(doc.entries.map((e) => {
            'code': '${e.key}',
            'text': '${e.key}',
            'IO': e.key == 'id' ? "H" : "O"
          }));
    }
    return columns;
  }

  getRows(List<Map<String, dynamic>> docs) {
    var rows = List<dynamic>.empty(growable: true);

    for (var i = 0; i < docs.length; i++) {
      var docData = docs[i];
      rows.add(docData);
    }
    return rows;
  }
}
